@@ -18,6 +18,7 @@
 #include "util.h"
 #include "log.h"
 #include "cpu.h"
+#include "files.h"
 #include "pstree.h"
 #include "cr-service.h"
 #include "cr-service-const.h"
@@ -162,6 +163,7 @@ int send_criu_rpc_script(enum script_actions act, char *name, int fd)
 
 	switch (act) {
 	case ACT_SETUP_NS:
+	case ACT_POST_RESTORE:
 		/*
 		 * FIXME pid is required only once on
 		 * restore. Need some more sane way of
@@ -344,6 +346,15 @@ static int setup_opts_from_req(int sk, CriuOpts *req)
 			goto err;
 	}
 
+	if (req->n_inherit_fd && !opts.swrk_restore) {
+		pr_err("inherit_fd is not allowed in standalone service\n");
+		goto err;
+	}
+	for (i = 0; i < req->n_inherit_fd; i++) {
+		if (inherit_fd_add(req->inherit_fd[i]->fd, req->inherit_fd[i]->key))
+			goto err;
+	}
+
 	for (i = 0; i < req->n_cg_root; i++) {
 		if (new_cg_root_add(req->cg_root[i]->ctrl,
 					req->cg_root[i]->path))
