@@ -6,7 +6,7 @@
 #include "asm/types.h"
 #include "signalfd.h"
 #include "proc_parse.h"
-#include "crtools.h"
+#include "imgset.h"
 #include "image.h"
 #include "util.h"
 #include "log.h"
@@ -20,9 +20,9 @@ struct signalfd_info {
 	struct file_desc	d;
 };
 
-int is_signalfd_link(int lfd)
+int is_signalfd_link(char *link)
 {
-	return is_anon_link_type(lfd, "[signalfd]");
+	return is_anon_link_type(link, "[signalfd]");
 }
 
 struct signalfd_dump_arg {
@@ -45,7 +45,7 @@ static int dump_signalfd_entry(union fdinfo_entries *e, void *arg)
 	e->sfd.flags = da->p->flags;
 	e->sfd.fown = (FownEntry *)&da->p->fown;
 
-	return pb_write_one(fdset_fd(glob_fdset, CR_FD_SIGNALFD),
+	return pb_write_one(img_from_set(glob_imgset, CR_FD_SIGNALFD),
 			&e->sfd, PB_SIGNALFD);
 }
 
